@@ -192,6 +192,7 @@ adbc_connection_release <- function(connection) {
 #' @param column_name Only show columns with the given name. If NULL, do not
 #'   filter by name. May be a search pattern.
 #' @param serialized_partition The partition descriptor.
+#' @param value A string or identifier.
 #'
 #' @return
 #'   - `adbc_connection_get_info()`, `adbc_connection_get_objects()`,
@@ -226,8 +227,8 @@ adbc_connection_get_info <- function(connection, info_codes) {
 
 #' @rdname adbc_connection_get_info
 #' @export
-adbc_connection_get_objects <- function(connection, depth, catalog, db_schema,
-                                        table_name, table_type, column_name) {
+adbc_connection_get_objects <- function(connection, depth = 0L, catalog = NULL, db_schema = NULL,
+                                        table_name = NULL, table_type = NULL, column_name = NULL) {
   error <- adbc_allocate_error()
   out_stream <- nanoarrow::nanoarrow_allocate_array_stream()
   status <- .Call(
@@ -302,6 +303,30 @@ adbc_connection_commit <- function(connection) {
   invisible(connection)
 }
 
+#' @rdname adbc_connection_get_info
+#' @export
+adbc_connection_quote_identifier <- function(connection, value, ...) {
+  UseMethod("adbc_connection_quote_identifier")
+}
+
+#' @rdname adbc_connection_get_info
+#' @export
+adbc_connection_quote_string <- function(connection, value, ...) {
+  UseMethod("adbc_connection_quote_string")
+}
+
+#' @export
+adbc_connection_quote_identifier.default <- function(connection, value, ...) {
+  out <- gsub('"', '""', enc2utf8(value))
+  paste0('"', out, '"')
+}
+
+#' @export
+adbc_connection_quote_string.default <- function(connection, value, ...) {
+  out <- gsub("'", "''", enc2utf8(value))
+  paste0("'", out, "'")
+}
+
 #' @rdname adbc_connection_get_info
 #' @export
 adbc_connection_rollback <- function(connection) {
